package filters;

use modules::semantics;

use strict;

sub filterCandidates
{
    my $candidates = $_[0];
    my $document = $_[1];
    my $sentence_target = $_[2];
    my $token_idx = $_[3];
    my $token_target = $_[4];

    my $filtered_candidates;

    foreach my $candidate (@{ $candidates })
    {
	my $filter = 0;
	if (&explicitFilter($candidate,$document,$sentence_target,$token_idx))
	{
	    $filter = 1;
	}
	elsif (&implicitFilter($candidate,$document,$sentence_target,$token_idx))
	{
	    $filter = 1;
	}
 	elsif (&domainFilter($candidate,$document,$sentence_target,$token_target))
 	{
 	    $filter = 1;
 	}

	if (not($filter))
	{
	    push @{ $filtered_candidates },$candidate;
	}
    }

    return $filtered_candidates;
}



sub explicitFilter
{
    my $candidate = $_[0];
    my $document = $_[1];
    my $sentence_target = $_[2];
    my $token_idx = $_[3];

    my $is_explicit=0;

    if ($sentence_target eq $candidate->{sentence})
    {
	my $target = $document->{$sentence_target}->{predicates}->{$token_idx};
 	foreach my $arg (keys %{ $target->{explicit_arg} })
 	{
	    my $filler = $document->{$sentence_target}->{predicates}->{$token_idx}->{explicit_arg}->{$arg}->{token};
	    
	    my $proper_span = span::properSpan($filler,$document->{$sentence_target});
	    my @span_tokens = split(/ /,$proper_span);
	    
	    foreach my $token (@span_tokens)
	    {
		if ($token eq $candidate->{token})
		{
		    $is_explicit = 1;
		}
	    }
 	}
    }
    

    return $is_explicit;
}


sub implicitFilter
{
    my $candidate = $_[0];
    my $document = $_[1];
    my $sentence_target = $_[2];
    my $token_idx = $_[3];

    my $is_implicit = 0;

    my $target = $document->{$sentence_target}->{predicates}->{$token_idx};

    foreach my $arg (keys %{ $target->{implicit_arg} })
    {
	my $filler_sentence = $document->{$sentence_target}->{predicates}->{$token_idx}->{implicit_arg}->{$arg}->{sentence};
	my $filler_token = $document->{$sentence_target}->{predicates}->{$token_idx}->{implicit_arg}->{$arg}->{token};

	if ($filler_sentence eq $candidate->{sentence})
	{
	    my $proper_span = span::properSpan($filler_token,$document->{$filler_sentence});
	    my @span_tokens = split(/ /,$proper_span);
	    
	    foreach my $token (@span_tokens)
	    {
		if ($token eq $candidate->{token})
		{
		    $is_implicit = 1;
		}
	    }
	}
    }

    return $is_implicit;
}



sub dependencyPath
{
    my $document = $_[0];
    my $sentence = $_[1];
    my $token = $_[2];

    my $path;

    my $head = $token;
    my $dep_relation = $document->{$sentence}->{tokens}->{$head}->{pdeprel};
    push @{ $path },$head;

    while ($dep_relation ne "ROOT")
    {
	$head = $document->{$sentence}->{tokens}->{$head}->{phead};
	$dep_relation = $document->{$sentence}->{tokens}->{$head}->{pdeprel};
	push @{ $path },$head;
    }

    return $path;
}



sub domainFilter
{
    my $candidate = $_[0];
    my $document = $_[1];
    my $sentence_target = $_[2];
    my $token_target = $_[3];

    my $domained = 0;

    if ($sentence_target eq $candidate->{sentence})
    {
	my $target_path = &dependencyPath($document,$sentence_target,$token_target);
	my $candidate_path = &dependencyPath($document,$candidate->{sentence},$candidate->{token});

	foreach my $node (@{ $candidate_path })
	{
	    if ($node eq $token_target)
	    {
		$domained = 1;
	    }
	}
	foreach my $node (@{ $target_path })
	{
	    if ($node eq $candidate->{token})
	    {
		$domained = 1;
	    }
	}
    }
 
    return $domained;
}



sub matchCategory
{
    my $candidates = $_[0];
    my $document = $_[1];
    my $predicate = $_[2];
    my $argument = $_[3];
    my $arg_categories = $_[4];

    my $filtered_candidates;

    my $categories;
    if (not exists( $arg_categories->{$predicate}->{$argument} ))
    {
	@{ $categories } = ("COGNITIVE","TANGIBLE","ACT","MONEY");
    }
    else
    {
	@{ $categories } = @{ $arg_categories->{$predicate}->{$argument} };
    }

    foreach my $candidate (@{ $candidates })
    {
	my $candidate_sentence = $candidate->{sentence};
	my $candidate_token = $candidate->{token};
	my $candidate_lemma = $document->{$candidate_sentence}->{tokens}->{$candidate_token}->{plemma};
	my $candidate_pos = $document->{$candidate_sentence}->{tokens}->{$candidate_token}->{ppos};
	my $candidate_ne = $document->{$candidate_sentence}->{tokens}->{$candidate_token}->{name_entity};
	my $candidate_ss = $document->{$candidate_sentence}->{tokens}->{$candidate_token}->{super_sense};
	    
	if (semantics::correctCategory($categories,$candidate_lemma,$candidate_pos,$candidate_ne,$candidate_ss))
	{
	    push @{ $filtered_candidates },$candidate;
	}
    }

    return $filtered_candidates;
}


1;
